#include "MinJerkSpline.h"

#define NCONTROL 6 // number of control points
#define NKNOTS 3 // start,via,end
#define NVARS 3 // number of variables to be optimized
#define GRADMIN 0.01 // minimum gradient
#define STARTGRAD 0.01
#define NPARAMS 5

MinJerkSpline::MinJerkSpline(double *X, double *Y, int NC, int NPoints)
{// constructor
	
	double *Params;// x1,vx1,x2,x3,vx2 or y1,vy1,y2,y3,vy2
	double *XVals;
	double *YVals;
	double a;
	double b;
	double c;
	double d;
	double e;
	double f;

	Params = new double[NPARAMS];
	XVals = new double[NVARS];
	YVals = new double[NVARS];
	ControlPoints = new Coord[NKNOTS - 1]; //initialize arrays

	for(int i = 0;i < (NKNOTS - 1);i++)
	{
		ControlPoints[i].x = new double[NCONTROL];
		ControlPoints[i].y = new double[NCONTROL];
	}

	Fill(X, Y, NC);// fille in Control from Via
	
	Params[0] = ControlPoints[0].x[0];
	Params[1] = ControlPoints[0].x[1];
	Params[2] = ControlPoints[0].x[5];
	Params[3] = ControlPoints[1].x[4];
	Params[4] = ControlPoints[1].x[5];

	XVals = Solve(Params);

	Params[0] = ControlPoints[0].y[0];
	Params[1] = ControlPoints[0].y[1];
	Params[2] = ControlPoints[0].y[5];
	Params[3] = ControlPoints[1].y[4];
	Params[4] = ControlPoints[1].y[5];

	YVals = Solve(Params);

	ControlPoints[0].x[2] = XVals[0];
	ControlPoints[0].x[3] = XVals[1];
	ControlPoints[0].x[4] = XVals[2];

	ControlPoints[0].y[2] = YVals[0];
	ControlPoints[0].y[3] = YVals[1];
	ControlPoints[0].y[4] = YVals[2];

	a = ControlPoints[0].x[0];
	b = ControlPoints[0].x[1];
	c = ControlPoints[0].x[2];
	d = ControlPoints[0].x[3];
	e = ControlPoints[0].x[4]; 
	f = ControlPoints[0].x[5];

	ControlPoints[1].x[0] = f;
	ControlPoints[1].x[1] = -e+2*f;
	ControlPoints[1].x[2] = d-4*e+4*f;
	ControlPoints[1].x[3] = -c+6*d-12*e+8*f;

	a = ControlPoints[0].y[0];
	b = ControlPoints[0].y[1];
	c = ControlPoints[0].y[2];
	d = ControlPoints[0].y[3];
	e = ControlPoints[0].y[4]; 
	f = ControlPoints[0].y[5];

	ControlPoints[1].y[0] = f;
	ControlPoints[1].y[1] = -e+2*f;
	ControlPoints[1].y[2] = d-4*e+4*f;
	ControlPoints[1].y[3] = -c+6*d-12*e+8*f;

}

Coord *MinJerkSpline::GetSplinePoints()
{
	int i = 0;
	int j = 0;
	Coord *result = new Coord[NKNOTS - 1];
	
	for(i = 0;i < (NKNOTS - 1);i++)
	{
		result[i].x = new double[NCONTROL];
		result[i].y = new double[NCONTROL];
	}

	for(i = 0;i < (NKNOTS - 1);i++)
		for(j = 0;j < NCONTROL;j++)
		{
			result[i].x[j] = ControlPoints[i].x[j];
			result[i].y[j] = ControlPoints[i].y[j];
		}

	return(result);
}

void MinJerkSpline::Fill(double *X, double *Y, int NC)
{
	int i = 0;
	int j = 0;

	for(i = 0;i < NCONTROL;i++)
		for(int j = 0;j < (NKNOTS - 1);j++)
		{ // set values to 0
			ControlPoints[j].x[i] = 0;
			ControlPoints[j].y[i] = 0;
		}


	if(NC == 3)
	{
		ControlPoints[0].x[0] = X[0];
		ControlPoints[0].y[0] = Y[0];
		ControlPoints[0].x[1] = X[0]; // speed is zero at start
		ControlPoints[0].y[1] = Y[0];

		ControlPoints[0].x[5] = X[1];
		ControlPoints[0].y[5] = Y[1];

		ControlPoints[1].x[4] = X[2]; // speed is zero at end
		ControlPoints[1].y[4] = Y[2];
		ControlPoints[1].x[5] = X[2];
		ControlPoints[1].y[5] = Y[2];
	} else if(NC == 5)
	{
		ControlPoints[0].x[0] = X[0];
		ControlPoints[0].y[0] = Y[0];
		ControlPoints[0].x[1] = (X[0] + X[3]); // speed input
		ControlPoints[0].y[1] = (Y[0] + Y[3]);


		ControlPoints[0].x[5] = X[1];
		ControlPoints[0].y[5] = Y[1];

		ControlPoints[1].x[4] = (X[2] - X[4]); //  speed input ( - Y[4] <- ? )
		ControlPoints[1].y[4] = (Y[2] - Y[4]);
		ControlPoints[1].x[5] = X[2];
		ControlPoints[1].y[5] = Y[2];

	} 
}

double MinJerkSpline::func(const gsl_vector *Vec, void *Param)
{
	double *P = (double *)Param;

	double a = P[0];
	double b = P[1];
	double c = gsl_vector_get(Vec, 0);
	double d = gsl_vector_get(Vec, 1);
	double e = gsl_vector_get(Vec, 2);
	double f = P[2];
	double m = P[3];
	double o = P[4];

	double result=
			(
				3*a*a
				+20*b*b
				+30*c*c
				-180*c*d
				+510*d*d
				+320*c*e
				-1920*d*e
				+1840*e*e
				+5*b*(-6*c+e)
				-200*c*f
				+1200*d*f
				-2320*e*f
				+736*f*f
				-a*(15*b-10*c+f)
				+30*c*m
				-180*d*m
				+355*e*m
				-230*f*m
				+20*m*m
				-10*c*o
				+60*d*o
				-120*e*o
				+79*f*o
				-15*m*o
				+3*o*o
			);
	return (result);
}


void MinJerkSpline::func_df(const gsl_vector *Vec, void *Param, gsl_vector *dVec)
{
	double *P = (double *)Param;

	double a = P[0];
	double b = P[1];
	double c = gsl_vector_get(Vec, 0);
	double d = gsl_vector_get(Vec, 1);
	double e = gsl_vector_get(Vec, 2);
	double f = P[2];
	double m = P[3];
	double o = P[4];

	double dc = 10*a 
				-30*b 
				+60*c 
				-180*d 
				+320*e 
				-200*f 
				+30*m 
				-10*o;

	double dd = -180*c 
				+1020*d 
				-1920*e 
				+1200*f 
				-180*m 
				+60*o;

	double de = 5*b 
				+320*c 
				-1920*d 
				+3680*e 
				-2320*f 
				+355*m 
				-120*o;

	gsl_vector_set(dVec, 0, dc);
	gsl_vector_set(dVec, 1, dd);
	gsl_vector_set(dVec, 2, de);
}

void MinJerkSpline::func_fdf(const gsl_vector *x, void *params, double *f, gsl_vector *df) 
{
  *f =func(x, params); 
  func_df(x, params, df);
}

double *MinJerkSpline::Solve(double *Params)
{
	size_t iter = 0;
    int status = 0;
	int i = 0;
	double *S;
	
	const gsl_multimin_fdfminimizer_type *T;
	gsl_multimin_fdfminimizer *s;
    gsl_multimin_function_fdf mf;
    gsl_vector *Start;

    mf.f = &func; // dunno how to point to member that is not static ... 
    mf.df = &func_df;
    mf.fdf = &func_fdf;
    mf.n = NVARS;
    mf.params = (void*)Params;

    Start = gsl_vector_alloc(NVARS);
	S = new double[NVARS];
	
	/* Seed the random-number generator with current time so 
	that the numbers will be different every time we run.*/
	srand( (unsigned)time( NULL ) );

	for(i = 0;i < NVARS;i++ )
	{
		 gsl_vector_set(Start, i, (double)(rand()/320));
	}

    T = gsl_multimin_fdfminimizer_vector_bfgs;
    s = gsl_multimin_fdfminimizer_alloc (T, NVARS);

    gsl_multimin_fdfminimizer_set(s, &mf, Start, STARTGRAD, 1e-4);

//	cout << "#Starting params:";
//	for(i = 0;i < NPARAMS;i++) cout << Params[i] << " ";
//	cout << "\tStartvec:";
//	for(i = 0;i < NVARS;i++)cout << gsl_vector_get(s->x, i) << " ";
//	cout << "\n";

    do
      {
		if(iter==1) cout << s->f << " ******* Cost function for min jerk spline at start" << endl;

        iter++;
        status = gsl_multimin_fdfminimizer_iterate(s);				

        if (status)
            break;

        status = gsl_multimin_test_gradient(s->gradient, GRADMIN);

        if (status == GSL_SUCCESS)
		{
			for(i = 0;i < NVARS;i++)
			{
				S[i] = gsl_vector_get(s->x, i);
				//cout << gsl_vector_get(s->x, i) << " ";
			}cout << s->f << " ******* Cost function for min jerk spline at end, minimum found" << endl;
		}


      }
    while (status == GSL_CONTINUE && iter < 30);

    gsl_multimin_fdfminimizer_free(s);
    gsl_vector_free(Start);

	return S;
}