#include "Spline.h"
#include "MinJerkSpline.h"
#include "pathmodel.h"
#include <iostream>
#define NKNOTS 3
#define NCONST 6
#define NPOINTS 50
#define NORDER 6

using namespace std;

void write_output(char *fname, double *x, double *y, int nsamples) 
{
   FILE *fp;
   int i;
   
   fp = fopen(fname, "w");
   for(i = 0; i < nsamples; ++i) 
   {
      fprintf(fp,"%f %f\n", x[i], y[i]);
   }
   fclose(fp);
}



void write_output(char *fname, double *x, int nsamples) 
{
	FILE *fp;
	int i;
   
	double *temp= new double[nsamples];

	for(i = 1; i < nsamples; i++)
	{
		temp[i] = static_cast<double>(i)/100;
	}

	fp = fopen(fname, "w");
	for(i = 0; i < nsamples; i++) 
	{
      fprintf(fp,"%f %f\n", temp[i], x[i]);
	}
	fclose(fp);
}

void write_data(char *fname, PathModel *Path, Coord *ControlPoints)
{
	FILE *fp;
	int i = 0;

	fp = fopen(fname, "w");


	fprintf(fp,"x points :\n");
	for(i = 0;i < NCONST;i++)
	{
		fprintf(fp,"%f\n",ControlPoints[0].x[i]);
	}
	for(i = 0;i < NCONST;i++)
	{
		fprintf(fp,"%f\n",ControlPoints[1].x[i]);
	}

	fprintf(fp,"\ny points :\n");
	for(i = 0;i < NCONST;i++)
	{
		fprintf(fp,"%f\n",ControlPoints[0].y[i]);
	}
	for(i = 0;i < NCONST;i++)
	{
		fprintf(fp,"%f\n",ControlPoints[1].y[i]);
	} 
	fprintf(fp,"\n");

	RcRcModel result;
	result = Path->GetAgoR();
	fprintf(fp,"Agonist Right Values:\n");
	fprintf(fp,"T1 : %f\n", result.T1);
	fprintf(fp,"T2 : %f\n", result.T2);
	fprintf(fp,"Amp: %f\n", result.Amp);

	result = Path->GetAgoL();
	fprintf(fp,"Agonist Left Values:\n");
	fprintf(fp,"T1 : %f\n", result.T1);
	fprintf(fp,"T2 : %f\n", result.T2);
	fprintf(fp,"Amp: %f\n", result.Amp);

	result = Path->GetAntR();
	fprintf(fp,"Antagonist Right Values:\n");
	fprintf(fp,"T1 : %f\n", result.T1);
	fprintf(fp,"T2 : %f\n", result.T2);
	fprintf(fp,"Amp: %f\n", result.Amp);

	result = Path->GetAntL();
	fprintf(fp,"Antagonist Left Values:\n");
	fprintf(fp,"T1 : %f\n", result.T1);
	fprintf(fp,"T2 : %f\n", result.T2);
	fprintf(fp,"Amp: %f\n", result.Amp);

	fclose(fp);
}

void main(int argv, char **argc)
{
	int i;
	double *x = new double[NCONST];
	double *y = new double[NCONST];
	double Angle = 0; // Angle begin spline, set as start for path generator
	char filename[80];
	char approx[80];
	char spline[80];
	char point[2];

	if(argv == (NCONST + 1)) 
	{
		x[0] = atof(argc[1]);
		y[0] = atof(argc[2]);
		x[1] = atof(argc[3]);
		y[1] = atof(argc[4]);
		x[2] = atof(argc[5]);
		y[2] = atof(argc[6]);
	} else 
	{
		cout << "input wrong\n";
		cout << "#usage: bfgs x1 y1 x2 y2 x3 y3" << endl;
		exit(0);
	}

	itoa(x[0],point,10);
	strcpy( filename, point);
	itoa(y[0],point,10);
	strcat( filename, point);
	itoa(x[1],point,10);
	strcat( filename, point);
	itoa(y[1],point,10);
	strcat( filename, point);
	itoa(x[2],point,10);
	strcat( filename, point);
	itoa(y[2],point,10);
	strcat( filename, point);

	MinJerkSpline Test(x, y, 3, 100); 
	Coord *Out;

	Out = new Coord[2];
	Out = Test.GetSplinePoints();

	double XOut1[NPOINTS];
	double YOut1[NPOINTS];
	Spline TestSpline1(NPOINTS, NORDER, &Out[0]);
	TestSpline1.GetX(XOut1);
	TestSpline1.GetY(YOut1);

	double XOut2[NPOINTS];
	double YOut2[NPOINTS];
	Spline TestSpline2(NPOINTS, NORDER, &Out[1]);
	TestSpline2.GetX(XOut2);
	TestSpline2.GetY(YOut2);

	double *X = new double[2*NPOINTS];	
	double *Y = new double[2*NPOINTS];	

	for(i = 0;i < NPOINTS;i++)
	{
		X[i] = XOut1[i];
		Y[i] = YOut1[i];
		X[i + NPOINTS] = XOut2[i+1];
		Y[i + NPOINTS] = YOut2[i+1];
	}

	Angle = atan2((YOut1[1]-YOut1[0]),(XOut1[1]-XOut1[0]));

	strcpy( spline, filename);
	strcat( spline,"spline.txt");	

	write_output(spline,X, Y, NPOINTS*2-1);
	cout << "# " <<  spline << ": saved minimum jerk spline for coordinates." << endl;

	PathModel Path(X ,Y, Angle,NPOINTS*2);
	SpeedProfile testspeed = Path.GetPath();

	strcpy( approx, filename);
	strcat( approx,"muscleR.txt");	

	write_output(approx,testspeed.Vr, NPOINTS*2);
	cout << "# " << approx << ": saved visco elastic Right model's approximation." << endl;

	strcpy( approx, filename);
	strcat( approx,"muscleL.txt");	

	write_output(approx,testspeed.Vl, NPOINTS*2);
	cout << "# " << approx << ": saved visco elastic Left model's approximation." << endl;

	strcpy( approx, filename);
	strcat( approx,"muscle.txt");	

	for(i = 0;i < NPOINTS*2;i++)
	{
	//	cout	<< X[i] << "\t" << Y[i] << "\t"
	//			<< testspeed.X[i] << "\t" << testspeed.Y[i] << endl;
	}

	write_output(approx,testspeed.X, testspeed.Y, NPOINTS*2);
	cout << "# " << approx << ": saved visco elastic model's approximation." << endl;


	double *a = new double [2*NPOINTS];
	double *s = new double [2*NPOINTS];

	char speeda[80];
	char speeds[80];

	for(i = 1;i < NPOINTS*2;i++)
	{
		s[i] = sqrt((X[i]-X[i-1])*(X[i]-X[i-1])+(Y[i]-Y[i-1])*(Y[i]-Y[i-1]));
		a[i] = sqrt((testspeed.X[i]-testspeed.X[i-1])*(testspeed.X[i]-testspeed.X[i-1])+(testspeed.Y[i]-testspeed.Y[i-1])*(testspeed.Y[i]-testspeed.Y[i-1]));
	}

	strcpy( speeda, filename);
	strcat( speeda,"speedmuscle.txt");	

	write_output(speeda,a,NPOINTS*2-1);
	cout << "# " <<  speeda << ": saved speed-profile for muscle model approximation." << endl;

	strcpy( speeds, filename);
	strcat( speeds,"speedspline.txt");	

	write_output(speeds,s,NPOINTS*2-1);
	cout << "# " <<  speeds << ": saved speed-profile for spline." << endl;	
	

	strcpy( speeds, filename);
	strcat( speeds,"data.txt");	


	write_data(speeds, &Path, Out);
	cout << "# " <<  speeds << ": saved Data concerning experiment." << endl;	

}