#include <gsl/gsl_multimin.h> 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#define NTARGETS 10 // there are six targets fo be fitted
#define NCOEFF 4 // there are four (a3, a2, a1, a0) to be fitted
#define STARTGRAD 0.001 // initial search gradient
#define GRADMIN 0.0001

using namespace std;

typedef struct
{
	double X[NTARGETS];
	double Y[NTARGETS];
} Data;



double P(const gsl_vector *B, double x) // function p(x) = B[0] + B[1]*X + ... + B[n]*X^n
{
	double result = 0;
	for(int i = 0;i < NCOEFF;i++)	result += gsl_vector_get(B, i) * pow(x, i);
	return result;
}
// Y wordt als vast gegeven meegenomen in de berekening van de afgeleide !
double Q(const gsl_vector *B, Data *Target) // function Q = Sum(Yi - P(Xi))^2
{
	double result = 0;

	for(int i = 0;i < NTARGETS;i++)
	{
		result += (Target->Y[i] - P(B, Target->X[i]))*(Target->Y[i] - P(B, Target->X[i]));
	}
	return result;
}

double func(const gsl_vector *B, void *Targets)
{
    Data *Target = (Data *)Targets;

	return Q(B, Target);
}


double XN(int N, Data *Target) // sums Xi^n for Xi in Target->X[i]
{
	double result = 0;
	for(int i = 0;i < NTARGETS;i++) result += pow(Target->X[i], N);
	return result;
}

double XNY(int N, Data *Target) // sums Xi^n*Yi for Xi, Yi in Target->X[i], Target->Y[i]
{
	double result = 0;
	for(int i = 0;i < NTARGETS;i++) result += pow(Target->X[i], N) * Target->Y[i];
	return result;
}

double Grad(const gsl_vector *BVec, int I, Data *Target)
{
	double result;

	for(int i = 0;i < NCOEFF;i++)
	{
		result += gsl_vector_get(BVec, i) * XN(i + I, Target);
	}
	result -= XNY(I, Target);
	return result;
}

void func_df (const gsl_vector *BVec, void *Targets, gsl_vector *dQdB)
{
    Data *Target = (Data *)Targets;

	double B[NCOEFF];

	for(int i = 0;i < NCOEFF;i++) B[i] = gsl_vector_get(BVec, i);

	for(i = 0;i < NCOEFF;i++)
	{
		gsl_vector_set(dQdB, i, Grad(BVec, i, Target));
	}
}

void func_fdf (const gsl_vector *x, void *params, double *f, gsl_vector *df) 
{
  *f =func(x, params); 
  func_df(x, params, df);
}

void InitData(Data *Target, const double *XVals, const gsl_vector *B)
{
	int i = 0;
	cout << "X\tY\n";
	cout << "--------------------\n";
/*	for(i = 0;i < NTARGETS;i++)
	{
		Target->X[i] = XVals[i];		
		Target->Y[i] = P(B, Target->X[i]);
	}*/

	
	Target->X[0] = 0;
	Target->X[1] = 1;
	Target->X[2] = 2;
	Target->X[3] = 3;
	Target->X[4] = 4;
	Target->X[5] = 5;
	Target->X[6] = 6;
	Target->X[7] = 7;
	Target->X[8] = 8;
	Target->X[9] = 9;

	Target->Y[0] = 10;
	Target->Y[1] = 34;
	Target->Y[2] = 172;
	Target->Y[3] = 676;
	Target->Y[4] = 1966;
	Target->Y[5] = 4630;
	Target->Y[6] = 9424;
	Target->Y[7] = 17272;
	Target->Y[8] = 29266;
	Target->Y[9] = 46666;

/*	Target->X[0] = 3;
	Target->X[1] = 4;
	Target->X[2] = 5;
	Target->X[3] = 6;
	Target->X[4] = 7;
	Target->X[5] = 8;

	Target->Y[0] = 676;
	Target->Y[1] = 1966;
	Target->Y[2] = 4630;
	Target->Y[3] = 9424;
	Target->Y[4] = 17272;
	Target->Y[5] = 29266;*/

	for(i = 0;i < NTARGETS;i++)
	{
		cout << Target->X[i] << "\t" << Target->Y[i] << endl;
	}
}

void PrintVec(gsl_vector *Vec, char *name)
{
	cout << name << ": ";
	for(int i = 0;i < NCOEFF;i++) cout << gsl_vector_get(Vec, i) << " ";
	cout << endl;
}

int main(int argv, char **argc)
{
    size_t iter = 0;
    int status = 0;
	int i = 0;

    const gsl_multimin_fdfminimizer_type *T;
    gsl_multimin_fdfminimizer *s;
    gsl_multimin_function_fdf mf;
    gsl_vector *StartB;
    gsl_vector *Solution;

	double TargetX[NTARGETS];

	Data *Target = new Data;

    mf.f = &func;
    mf.df = &func_df;
    mf.fdf = &func_fdf;
    mf.n = NCOEFF;
    mf.params = (void*)Target;

    StartB = gsl_vector_alloc(NCOEFF);
    Solution = gsl_vector_alloc(NCOEFF);
	
	cout << "usage: bfgs start[1], ... ,start[n], target[0], ... ,target[n]" << ",  n = "<< NCOEFF << endl;

	if(argv == (2 * NCOEFF + 1)) 
	{
		for(i = 0		;i < NCOEFF		;i++) gsl_vector_set(StartB, i, atof(argc[i + 1]));
		for(i = NCOEFF	;i < (2* NCOEFF);i++) gsl_vector_set(Solution, i - NCOEFF, atof(argc[i + 1]));
	}	else 
	{
		for(i = 0;i < NCOEFF;i++) gsl_vector_set(StartB, i, 1);
		for(i = 0;i < NCOEFF;i++) gsl_vector_set(Solution, i, 2);
	}

//	PrintVec(Solution, "#solution vec");
	PrintVec(StartB, "#start vec   ");	

	for(i = 0;i < NTARGETS;i++) TargetX[i] = i;
	InitData(Target, TargetX, Solution);

    T = gsl_multimin_fdfminimizer_vector_bfgs;
    s = gsl_multimin_fdfminimizer_alloc (T, NCOEFF);

    gsl_multimin_fdfminimizer_set(s, &mf, StartB, STARTGRAD, 1e-4);

    do
      {

        iter++;
        status = gsl_multimin_fdfminimizer_iterate(s);				

        if (status)
            break;

        status = gsl_multimin_test_gradient(s->gradient, GRADMIN);

        if (status == GSL_SUCCESS)
		{
            printf ("Minimum found at:\n");
		}
		
		cout << "Obj Fn: " << s->f << "\t";
		PrintVec(s->x, "Vec");
      }
    while (status == GSL_CONTINUE && iter < 200);

    gsl_multimin_fdfminimizer_free(s);
    gsl_vector_free(StartB);
    gsl_vector_free(Solution);

    return 0;
}
/*


	gsl_vector_set(Solution, 0, 1);
	gsl_vector_set(Solution, 1, 1);


	cout << "#usage: bfgs A0, ... ,An (doubles)\n";

	cout << "#solution input:\t";
	for(i  = 0;i < NCOEFF;i++) cout << gsl_vector_get(Solution, i) << "\t";
	cout << endl << endl;

	if(argv > NCOEFF) 
	{
		for(i = 0;i < NCOEFF;i++) gsl_vector_set(StartB, i, atof(argc[i + 1]));
	}	else for(int i = 0;i < NCOEFF;i++) gsl_vector_set(StartB, i, 1);

	cout << "#Starting search using next formula:\n";
	cout << "#starting with:  \t";
	for(i  = 0;i < NCOEFF;i++) cout << gsl_vector_get(StartB, i) << "\t";


*/