#ifndef _PATHMODEL_H_
#define _PATHMODEL_H_

#include <gsl/gsl_multimin.h>
#include <time.h>
#include <iostream>
#include <math.h>


typedef struct
{
	double
		T1,	// first time constant
		T2,	// second ""
		Amp,	// Amplification
		LB,	// Low bound of integration range
		UB;// Upper bound of integration range
} RcRcModel;

typedef struct
{
	double 
		*Vr, // right wheel speeds
		*Vl, // left wheel speeds
		*Angle, // Angles for traject
		*X, // odometric x-positions for robot
		*Y, // doometric y-positions for robot
		dT; // Dt between speed commands
} SpeedProfile; // used to calculate fitness of a path

using namespace std;

class PathModel
{
public:
	PathModel(double *, double *, double, int);// x-data,y-data,start-angle,npoints
	RcRcModel		GetAgoR();
	RcRcModel		GetAgoL();
	RcRcModel		GetAntR();
	RcRcModel		GetAntL();
	SpeedProfile	GetPath();

private:
	static double	func(const gsl_vector *, void *);
	static void		func_df(const gsl_vector *, void *, gsl_vector *);
	static void		func_fdf(const gsl_vector *, void *, double *, gsl_vector *);
	
	void			Fill(); // Fills SpeedProfile given RcRcModel
	double			N; // number of points in the path
	double			StartAngle;
	double			Sqr();
	double			*XMJ; // min jerk points to fit, X
	double			*YMJ; // min jerk points to fit, Y

	SpeedProfile	Path; // path description for RcRcModel
	RcRcModel		AgoR; // Agonist, right
	RcRcModel		AgoL; // Agonist, left
	RcRcModel		AntR; // Antagonist, right
	RcRcModel		AntL; // Antagonist, left
	
	double			RcRc(double, double, double,double, double, double, double);
	void			CalcVec(double, double, double, double, double, double *, double *, double *);
	void			Solve();// solves for a Model and SpeedProfile
};

#endif