#include "spline.h"

Spline::Spline(int NPoints, int NOrder, Coord *VecInput)
{ // constructor
	double *PascalRow = 0;
	double	*DrX; // temp difference list for X
	double	*DrY; // temp difference list for Y

	N = NPoints;
	O = NOrder;
	x = new double[N];
	y = new double[N];

	Control = new Coord; // is this needed ? 
	Control->x = new double[O];
	Control->y = new double[O];

	P = new Coord;
	P->x = new double[O]; // parameters in spline for x coord
	P->y = new double[O]; // parameters in spline for y coord

	PascalRow = new double[O];
	DrX = new double[O];
	DrY = new double[O];

	for(int i = 0;i < N;i++) 
	{
		x[i] = 0;
		y[i] = 0;
	}

	for(i = 0;i < O;i++)
	{
		P->x[i] = 0;
		P->y[i] = 0;
		Control->x[i] = VecInput->x[i];
		Control->y[i] = VecInput->y[i];
		PascalRow[i] = 0;
		DrX[i] = 0;
		DrY[i] = 0;
	}

	Diff(P->x, Control->x);
	Diff(P->y, Control->y);
	
	calculated = false;

	Fill();
}

void Spline::Diff(double *D, double *Contr)
{
	double *PascalRow;
	double *PascalTotal;

	PascalRow = new double[O];
	PascalTotal = new double[O];

	Pascal(PascalTotal, O);

	for(int i = 0;i < O;i++)
	{
		Pascal(PascalRow, (i + 1));

		for(int j = 0;j < (i + 1);j++)
		{
			if(Odd(i))
			{
				if(Odd(j))
				{
					D[i] += PascalTotal[i]*PascalRow[j]*Contr[j]; 
				} else 
				{
					D[i] -= PascalTotal[i]*PascalRow[j]*Contr[j]; 
				}
			} else
			{
				if(Odd(j))
				{
					D[i] -= PascalTotal[i]*PascalRow[j]*Contr[j]; 
				} else
				{
					D[i] += PascalTotal[i]*PascalRow[j]*Contr[j]; 
				}
			}
		}
	}
}

void Spline::PrintResult(double *X, double *Y, int N)
{
	cout << "#x		f(x)\n";
	cout << "#----------------\n";

	for(int i = 0;i < N;i++)
	{
		cout << X[i] << "\t" << Y[i] << endl;
	}
	cout << endl;

}

int Spline::Fac(int N)
{
	if(N > 1)
	{
		return(Fac(N - 1)* N);
	} else return 1; 	
}

void Spline::Pascal(double *P, int O)
{ // fills P with pascal row O, row 1 = |1|, 2 = |1|1| etc
	int Order = O - 1;
	int FacOrder = Fac(Order);

	for(int pos = 0;pos < Order;pos++)
	{
		P[pos] = double(FacOrder/(Fac(Order - pos)*Fac(pos)));
	}

	P[O - 1] = 1;
}

int Spline::Odd(int inputval)
{
	return inputval & 1;
}


double Spline::Ft(double *Param, double T)
{

	double result = 0;
	for(int i = 0;i < O;i++) 
	{
		result += Param[i]*pow(T, i);
	}
	return result;
}

void  Spline::Fill()
{
	double dT	= 1/(double)(N - 1);
	double T = 0;

	for(int i = 0;i < N;i++)
	{
		x[i] = Ft(P->x, T) ; 
		y[i] = Ft(P->y, T) ; 
		T += dT; // take next time step
	}
}

void Spline::GetX(double *OutX)
{
	for(int i = 0;i < N;i++)
	{
		OutX[i] = x[i];
	}
}

void Spline::GetY(double *OutY)
{
	for(int i = 0;i < N;i++)
	{
		OutY[i] = y[i];
	}
}

